@@ -2,9 +2,9 @@
 /*! \file rational.h.in
  ** \verbatim
  ** Top contributors (to current version):
- **   Morgan Deters, Paul Meng, Tim King
+ **   Morgan Deters, Tim King
  ** This file is part of the CVC4 project.
- ** Copyright (c) 2009-2017 by the authors listed in the file AUTHORS
+ ** Copyright (c) 2009-2019 by the authors listed in the file AUTHORS
  ** in the top-level source directory) and their institutional affiliations.
  ** All rights reserved.  See the file COPYING in the top-level source
  ** directory for licensing information.\endverbatim
@@ -30,13 +30,13 @@
 #ifdef CVC4_CLN_IMP
 #  include <cvc4/util/rational_cln_imp.h>
 #  if SWIG
-     %include "util/rational_cln_imp.h"
+     %include <cvc4/util/rational_cln_imp.h>
 #  endif /* SWIG */
 #endif /* CVC4_CLN_IMP */
 
 #ifdef CVC4_GMP_IMP
 #  include <cvc4/util/rational_gmp_imp.h>
 #  if SWIG
-     %include "util/rational_gmp_imp.h"
+     %include <cvc4/util/rational_gmp_imp.h>
 #  endif /* SWIG */
 #endif /* CVC4_GMP_IMP */
